# -*- coding: utf-8 -*-
# @Author: gzliuxin
# @Email:  gzliuxin@corp.netease.com
# @Date:   2017-07-14 19:47:51

from poco.drivers.std import StdPoco
from poco.utils.device import VirtualDevice


__all__ = ['QtPoco']
DEFAULT_PORT = 9001
DEFAULT_ADDR = ("localhost", DEFAULT_PORT)


class QtPoco(StdPoco):
    """
    Poco Qt implementation.
    """

    def __init__(self, addr=DEFAULT_ADDR, **options):
        dev = VirtualDevice(addr[0])
        super(QtPoco, self).__init__(addr[1], dev, **options)



# coding=utf-8

from poco.pocofw import Poco
from poco.agent import PocoAgent
from poco.drivers.std.attributor import StdAttributor
from poco.drivers.std.dumper import StdDumper
from poco.drivers.std.screen import StdScreen
from poco.drivers.std.inputs import StdInput
from poco.freezeui.hierarchy import FrozenUIHierarchy
from poco.utils.airtest import AirtestInput
from poco.utils.simplerpc.rpcclient import RpcClient
from poco.utils.simplerpc.transport.tcp.main import TcpClient
from poco.utils.simplerpc.utils import sync_wrapper

from airtest.core.api import connect_device, device as current_device
from airtest.core.helper import device_platform
import socket


__all__ = ['StdPoco', 'StdPocoAgent']
DEFAULT_PORT = 15004
DEFAULT_ADDR = ('localhost', DEFAULT_PORT)


class StdPocoAgent(PocoAgent):
    def __init__(self, addr=DEFAULT_ADDR, use_airtest_input=True):
        self.conn = TcpClient(addr)
        self.c = RpcClient(self.conn)
        self.c.DEBUG = False
        self.c.connect()

        hierarchy = FrozenUIHierarchy(StdDumper(self.c), StdAttributor(self.c))
        screen = StdScreen(self.c)
        if use_airtest_input:
            inputs = AirtestInput()
        else:
            inputs = StdInput(self.c)
        super(StdPocoAgent, self).__init__(hierarchy, inputs, screen, None)

    @property
    def rpc(self):
        return self.c

    @sync_wrapper
    def get_debug_profiling_data(self):
        return self.c.call("GetDebugProfilingData")

    @sync_wrapper
    def get_sdk_version(self):
        return self.c.call('GetSDKVersion')


class StdPoco(Poco):
    """
    Poco standard implementation for PocoSDK protocol.

    Args:
        port (:py:obj:`int`): the port number of the server that listens on the target device. default to 15004.
        device (:py:obj:`Device`): :py:obj:`airtest.core.device.Device` instance provided by ``airtest``. leave the
         parameter default and the default device will be chosen. more details refer to ``airtest doc``
        options: see :py:class:`poco.pocofw.Poco`

    Examples:
        The simplest way to connect to a cocos2dx-lua game::

            from poco.drivers.std import StdPoco
            from airtest.core.api import connect_device

            # connect a device first, then initialize poco object
            device = connect_device('Android:///')
            poco = StdPoco(10054, device)

            # now you can play with poco
            ui = poco('...')
            ui.click()
            ...

    """

    def __init__(self, port=DEFAULT_PORT, device=None, use_airtest_input=True, **kwargs):
        self.device = device or current_device()
        if not self.device:
            self.device = connect_device("Android:///")

        platform_name = device_platform(self.device)
        if platform_name == 'Android':
            # always forward for android device to avoid network unreachable
            local_port, _ = self.device.adb.setup_forward('tcp:{}'.format(port))
            ip = self.device.adb.host or 'localhost'
            port = local_port
        elif platform_name == 'IOS':
            # ip = device.get_ip_address()
            # use iproxy first
            ip = 'localhost'
            port, _ = self.device.instruct_helper.setup_proxy(port)
        else:
            try:
                ip = self.device.get_ip_address()
            except AttributeError:
                try:
                    ip = socket.gethostbyname(socket.gethostname())
                except socket.gaierror:
                    # 某些特殊情况下会出现这个error，无法正确获取本机ip地址
                    ip = 'localhost'

        agent = StdPocoAgent((ip, port), use_airtest_input)
        kwargs['reevaluate_volatile_attributes'] = True
        super(StdPoco, self).__init__(agent, **kwargs)



# coding=utf-8

from poco.sdk.exceptions import UnableToSetAttributeException
from poco.sdk.AbstractNode import AbstractNode
from poco.utils.six import string_types


class OSXUINode(AbstractNode):

    def __init__(self, control, dumper):
        self.Control = control
        self.dumper = dumper

    def getParent(self):
        return OSXUINode(self.Control.AXParent, self.dumper)

    def getChildren(self):
        childs = self.Control.AXChildren
        if childs is not None:
            for node in childs:
                yield OSXUINode(node, self.dumper)

    def getAttr(self, attrName):

        attrs = self.Control.getAttributes()

        if attrName == 'name':
            if 'AXTitle' in attrs:
                if self.Control.AXTitle != "" and self.Control.AXTitle is not None:
                    return self.Control.AXTitle
            if 'AXRole' in attrs:
                return self.Control.AXRole[2:]

        if attrName == 'originType':
            if 'AXRole' in attrs:
                return self.Control.AXRole
            return "Unknow"

        if attrName == 'type':
            if 'AXRole' in attrs:
                return self.Control.AXRole[2:]
            return "Unknow"

        if attrName == 'pos':
            pos = self.Control.AXPosition
            size = self.Control.AXSize
            return [float(pos[0] + size[0] / 2.0 - self.dumper.RootLeft) / float(self.dumper.RootWidth), float(pos[1] + size[1] / 2.0 - self.dumper.RootTop) / float(self.dumper.RootHeight)]

        if attrName == 'size':
            pos = self.Control.AXPosition
            size = self.Control.AXSize
            return [size[0] / float(self.dumper.RootWidth), size[1] / float(self.dumper.RootHeight)]

        if attrName == 'text':
            if 'AXValue' in attrs:
                if isinstance(self.Control.AXValue, string_types):
                    return self.Control.AXValue
                if isinstance(self.Control.AXValue, int):
                    return str(self.Control.AXValue)
                if isinstance(self.Control.AXValue, float):
                    return str(self.Control.AXValue)
                return None

        return super(OSXUINode, self).getAttr(attrName)

    def setAttr(self, attrName, val):
        attrs = self.Control.getAttributes()
        if attrName != 'text':
            raise UnableToSetAttributeException(attrName, self)
        else:
            if 'AXValue' in attrs:
                self.Control.AXValue = val
            else:
                raise UnableToSetAttributeException(attrName, self)

    def getAvailableAttributeNames(self):
        if 'AXValue' in self.Control.getAttributes():
            return super(OSXUINode, self).getAvailableAttributeNames() + ('text', 'originType')
        else:
            return super(OSXUINode, self).getAvailableAttributeNames() + ('originType', )



# coding=utf-8

"""
**The brief introduction**:

This package (:py:mod:`poco.sdk.interfaces`) defines the main standards for communication interfaces between poco and
poco-sdk. If poco-sdk is integrated with an app running on another host or in different language, then poco-sdk is
called `remote runtime`. The implementation of these interfaces can be done either remotely or locally depending on
your own choice. If it is done locally, refer to :py:mod:`poco.freezeui` for more information.

Poco needs to communicate with the app runtime under the convention of interfaces described below and these interfaces
must be properly implemented. Any object implementing the same interface is replaceable and the communication protocol
or transport layer has no limitation. Furthermore, in many cases the communication can be customized that one part of
interfaces can use HTTP protocol and other can use TCP.

"""



# coding=utf-8

from poco.drivers.unity3d.test.tutorial.case import TutorialCase


class DragTutorial(TutorialCase):
    def runTest(self):
        self.poco('star').drag_to(self.poco('shell'))


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# coding=utf-8

import time
from poco.drivers.unity3d.test.tutorial.case import TutorialCase


class LocalPositioning1Tutorial(TutorialCase):
    def runTest(self):
        balloonfish_image = self.poco(text='balloonfish').focus([0.5, -3])
        balloonfish_image.long_click()
        time.sleep(0.2)


if __name__ == '__main__':
    import pocounit
    pocounit.main()



# coding=utf-8

"""
This module provides several exceptions for poco-sdk. These exceptions are raised only in sdk corresponding runtime.
"""


__author__ = 'lxn3032'
__all__ = ['NoSuchTargetException', 'NodeHasBeenRemovedException', 'UnableToSetAttributeException',
           'NoSuchComparatorException', 'NonuniqueSurfaceException', 'InvalidSurfaceException']


class NodeHasBeenRemovedException(Exception):
    """
    Raised when the node (UI element) is refreshed (updated, recycled or destroyed) while retrieving the attributes
    during traversing the hierarchy.

    In some engines implementations, the UI hierarchy is refreshed in a stand-alone thread while poco is performing
    the traverse process, so the engine error might be triggered when poco is trying to retrieve the attribute of
    the UI element but the attribute is being updated at the same time. In this situation, poco sdk catches the engine
    error and raises this exception.
    """

    def __init__(self, attrName, node):
        msg = 'Node was no longer alive when query attribute "{}". Please re-select.'.format(attrName)
        super(NodeHasBeenRemovedException, self).__init__(msg)


class UnableToSetAttributeException(Exception):
    """
    Raised when settings the attribute of the given UI element failed. In most cases, the reason why it failed is that
    the UI element does not support mutation. From the point of view of SDK implementation, this exception can be
    raised proactively to indicate that the modification of the attribute is not allowed.
    """

    def __init__(self, attrName, node):
        msg = 'Unable to set attribute "{}" of node "{}".'.format(attrName, node)
        super(UnableToSetAttributeException, self).__init__(msg)


class NoSuchTargetException(Exception):
    """
    Raised when the index is out of range for selecting the UI element by given index.
    
    .. TODO:: Maybe this is a little bit redundant to :py:class:`poco.exceptions.PocoNoSuchNodeException`. 
     Should be optimized.
    """

    pass


class NoSuchComparatorException(Exception):
    """
    Raised when the matcher does not support the given comparison method.
    """

    def __init__(self, matchingMethod, matcherName):
        super(NoSuchComparatorException, self).__init__()
        self.message = 'No such matching method "{}" of this Matcher ("{}")'.format(matchingMethod, matcherName)


class NonuniqueSurfaceException(Exception):
    """
    Raised when the device selector matches mutiple devices surface
    """

    def __init__(self, selector):
        msg = 'The arguments ("{}") match multiple device surface. More precise conditions required.'.format(selector)
        super(NonuniqueSurfaceException, self).__init__(msg)


class InvalidSurfaceException(Exception):
    """
    Raised when the device surface is invalid
    """

    def __init__(self, target, msg="None"):
        msg = 'Target device surface invalid ("{}") . Detial message: "{}"'.format(target, msg)
        super(InvalidSurfaceException, self).__init__(msg)




# coding=utf-8
from __future__ import unicode_literals, division

import math
import copy
import poco.utils.six as six
import time
from functools import wraps

from poco.gesture import PendingGestureAction
from poco.exceptions import PocoTargetTimeout, InvalidOperationException, PocoNoSuchNodeException, PocoTargetRemovedException
from poco.sdk.exceptions import UnableToSetAttributeException
from poco.utils.query_util import query_expr, build_query
from poco.utils.multitouch_gesture import make_pinching

__all__ = ['UIObjectProxy']


def wait(func):
    @wraps(func)
    def wrapped(proxy, *args, **kwargs):
        try:
            return func(proxy, *args, **kwargs)
        except PocoNoSuchNodeException as e:
            try:
                proxy.wait_for_appearance(timeout=proxy.poco._pre_action_wait_for_appearance)
                return func(proxy, *args, **kwargs)
            except PocoTargetTimeout:
                raise e

    return wrapped


def refresh_when(err_type):
    def wrapper(func):
        @wraps(func)
        def wrapped(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except err_type:
                self._do_query(multiple=False, refresh=True)
                return func(self, *args, **kwargs)
        return wrapped
    return wrapper


class ReevaluationContext(object):
    """
    volatile attributes的重运算同一批次内只执行一次，无需多次执行
    """

    def __init__(self, proxy):
        self.target = proxy
        self.with_this_batch = hasattr(self.target, '__reevaluation_context__')

    def __enter__(self):
        if not self.with_this_batch:
            setattr(self.target, '__reevaluation_context__', True)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if not self.with_this_batch:
            delattr(self.target, '__reevaluation_context__')


def volatile_attribute(func):
    @wraps(func)
    def wrapped(proxy, *args, **kwargs):
        with ReevaluationContext(proxy) as rc:
            if not rc.with_this_batch and proxy.poco._reevaluate_volatile_attributes:
                proxy._evaluated = False
            return func(proxy, *args, **kwargs)

    return wrapped


class UIObjectProxy(object):
    """
    UI Proxy class that represents the UI element on target device.

    Any action performing on this instance is handled by Poco. It is not necessary to initialize this object manually.
    See ``QueryCondition`` for more details about how to select the UI elements.

    Args:
        poco: the poco instance
        name: query condition of "name" attribute, i.e. the UI element(s) with ``name`` name will be selected
        attrs: other query expressions except for the ``name``

    See Also:
        :py:meth:`select UI element(s) by poco <poco.pocofw.Poco.__call__>`
    """

    def __init__(self, poco, name=None, **attrs):
        # query object in tuple
        self.query = build_query(name, **attrs)
        self.poco = poco

        # this flag is introduced to improve the performance, it is set if multiple UI elements are selected and
        # it does not affect the selection result
        # 上一次选择是否是多选，如果不是多选但需要访问所有UI elements时会进行重新选择。
        self._query_multiple = False

        # true or false whether the corresponding UI elements of this UI proxy (self) have been selected
        # 此UI proxy是否已经查找到对应的UI elements了
        self._evaluated = False

        # the proxy object of UI elements, migh be `node` or `[nodes]`, the proxy type is specified by
        # `self._nodes_proxy_is_list`
        # 可能是远程node代理，也可能是远程[node]代理, 由`self._nodes_proxy_is_list`指定是何种proxy类型
        self._nodes = None
        self._nodes_proxy_is_list = True

        # use only for caching some proxies of sorted nodes in `self.__getitem__`
        # 仅用于__getitem__时保存好已排序的child代理对象
        self._sorted_children = None

        # focus point of the UI element, see `CoordinateSystem` for more details
        # 相对于包围盒的focus point定义，用于touch/swipe/drag操作的局部相对定位
        self._focus = None

    def child(self, name=None, **attrs):
        """
        Select the direct child(ren) from the UI element(s) given by the query expression, see ``QueryCondition`` for
        more details about the selectors.

        Args:
            name: query expression of attribute "name", i.e. the UI elements with ``name`` name will be selected
            attrs: other query expression except for the ``name``

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a new UI proxy object representing the child(ren) of
            current UI element(s)
        """

        sub_query = build_query(name, **attrs)
        query = ('/', (self.query, sub_query))
        obj = UIObjectProxy(self.poco)
        obj.query = query
        return obj

    def children(self):
        """
        The same as :py:meth:`.child() <poco.proxy.UIObjectProxy.child>` but it selects all children from the UI
        element(s).

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a new UI proxy object
        """

        return self.child()

    def offspring(self, name=None, **attrs):
        """
        Select the offsprings including the direct child(ren) from the UI element(s) given by the query expression,
        see ``QueryCondition`` for more details about selectors.

        Args:
            name: query expression of attribute "name", i.e. the UI elements with ``name`` name will be selected
            attrs: other query expression except for the ``name``

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a new UI proxy object representing the child(ren) of
            current UI element(s)
        """

        sub_query = build_query(name, **attrs)
        query = ('>', (self.query, sub_query))
        obj = UIObjectProxy(self.poco)
        obj.query = query
        return obj

    def sibling(self, name=None, **attrs):
        """
        Select the sibling(s) from the UI element(s) given by the query expression, see ``QueryCondition`` for more
        details about the selectors.

        Args:
            name: query expression of attribute "name", i.e. the UI elements with ``name`` name will be selected
            attrs: other query expression except for the ``name``

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a new UI proxy object representing the child(ren) of
            current UI element(s)
        """

        sub_query = build_query(name, **attrs)
        query = ('-', (self.query, sub_query))
        obj = UIObjectProxy(self.poco)
        obj.query = query
        return obj

    def parent(self):
        """
        Select the direct child(ren) from the UI element(s) given by the query expression, see ``QueryCondition`` for
        more details about the selectors.

        Warnings:
            Experimental method, may not be available for all drivers.

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a new UI proxy object representing the direct parent
            of the first UI element.
        """

        sub_query = build_query(None)  # as placeholder
        query = ('^', (self.query, sub_query))
        obj = UIObjectProxy(self.poco)
        obj.query = query
        return obj

    def __getitem__(self, item):
        """
        Select the specific UI element by index. If this UI proxy represents a set of UI elements, then use this method
        to access the specific UI element. The new UI element will be wrapped by UIObjectProxy instance and therefore
        the returned value is also the UI proxy object.

        The order of UI elements are determined by their position on the screen and not by the selection sequence. This
        rule is called  "L2R U2D" (one by one from left to right, line by line from up to down), i.e. the most top left
        UI element is always the first one. See ``IterationOverUI`` for more details.

        Warnings:
            This method may cause some performance issues depending on implementation of PocoAgent.

        Args:
            item (:obj:`int`): the index.

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a new UI proxy object representing the n-th of the
            current UI elements.
        """

        if not self._query_multiple:
            nodes = self._do_query(multiple=True, refresh=True)
        else:
            nodes = self._nodes
        length = len(nodes)
        if not self._sorted_children:
            self._sorted_children = []
            for i in range(length):
                uiobj = UIObjectProxy(self.poco)
                uiobj.query = ('index', (self.query, i))
                uiobj._evaluated = True
                uiobj._query_multiple = True
                uiobj._nodes = nodes[i]
                uiobj._nodes_proxy_is_list = False
                pos = uiobj.get_position()
                self._sorted_children.append((uiobj, pos))

        self._sorted_children.sort(key=lambda v: (v[1][1], v[1][0]))
        return self._sorted_children[item][0]

    def __len__(self):
        """
        Return the number of selected UI elements.

        Returns:
            :obj:`int`: returns 0 if none of the UI element matches the query expression otherwise returns the number
            of selected UI elements
        """

        if not self._nodes_proxy_is_list:
            return 1

        # 获取长度时总是multiple的
        if not self._query_multiple:
            try:
                nodes = self._do_query(multiple=True, refresh=True)
            except PocoNoSuchNodeException:
                nodes = []
        else:
            nodes = self._nodes
        return len(nodes)

    def __iter__(self):
        """
        Similar method to :py:meth:`.__getitem__() <poco.proxy.UIObjectProxy.__getitem__>` with the difference that this
        method iterates over all UI elements. The order rules of UI elements is same as for :py:meth:`.__getitem__()
        <poco.proxy.UIObjectProxy.__getitem__>`. See ``IterationOverUI`` for more details.

        Yields:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a generator yielding new UI proxy represents the
            specific UI element iterated over

        Raises:
            PocoTargetRemovedException: when hierarchy structure has changed and it is attempted to access to the
             nonexistent UI element over the iteration
        """

        # 节点数量太多时，就不按照控件顺序排序了
        if not self._query_multiple:
            nodes = self._do_query(multiple=True, refresh=True)
        else:
            nodes = self._nodes
        length = len(nodes)
        sorted_nodes = []
        for i in range(length):
            uiobj = UIObjectProxy(self.poco)
            uiobj.query = ('index', (self.query, i))
            uiobj._evaluated = True
            uiobj._query_multiple = True
            uiobj._nodes = nodes[i]
            uiobj._nodes_proxy_is_list = False
            pos = uiobj.get_position()
            sorted_nodes.append((uiobj, pos))
        sorted_nodes.sort(key=lambda v: (v[1][1], v[1][0]))

        for obj, _ in sorted_nodes:
            yield obj

    @wait
    def click(self, focus=None, sleep_interval=None):
        """
        Perform the click action on the UI element(s) represented by the UI proxy. If this UI proxy represents a set of
        UI elements, the first one in the set is clicked and the anchor point of the UI element is used as the default
        one. It is also possible to click another point offset by providing ``focus`` argument.

        See ``CoordinateSystem`` for more details.

        Args:
            focus (2-:obj:`tuple`/2-:obj:`list`/:obj:`str`): an offset point (x, y) from the top left corner of the UI
             element(s), values must be in range of 0~1. This argument can be also specified by 'anchor' or 'center'.
             'Center' means to click the center of bounding box of UI element. 
            sleep_interval: number of seconds to wait after this action. Default is None which is the default sleep
             interval. This value can be configured by Poco initialization. See configuration at poco
             :py:class:`initialization <poco.pocofw.Poco>` for more details.

        Raises:
            PocoNoSuchNodeException: raised when the UI element does not exist
        """

        focus = focus or self._focus or 'anchor'
        pos_in_percentage = self.get_position(focus)
        self.poco.pre_action('click', self, pos_in_percentage)
        ret = self.poco.click(pos_in_percentage)
        if sleep_interval:
            time.sleep(sleep_interval)
        else:
            self.poco.wait_stable()
        self.poco.post_action('click', self, pos_in_percentage)
        return ret

    @wait
    def rclick(self, focus=None, sleep_interval=None):
        """
        Perform the right click action on the UI element(s) represented by the UI proxy. If this UI proxy represents a set of
        UI elements, the first one in the set is clicked and the anchor point of the UI element is used as the default
        one. It is also possible to click another point offset by providing ``focus`` argument.

        See ``CoordinateSystem`` for more details.

        Args:
            focus (2-:obj:`tuple`/2-:obj:`list`/:obj:`str`): an offset point (x, y) from the top left corner of the UI
             element(s), values must be in range of 0~1. This argument can be also specified by 'anchor' or 'center'.
             'Center' means to click the center of bounding box of UI element. 
            sleep_interval: number of seconds to wait after this action. Default is None which is the default sleep
             interval. This value can be configured by Poco initialization. See configuration at poco
             :py:class:`initialization <poco.pocofw.Poco>` for more details.

        Raises:
            PocoNoSuchNodeException: raised when the UI element does not exist
        """

        focus = focus or self._focus or 'anchor'
        pos_in_percentage = self.get_position(focus)
        self.poco.pre_action('rclick', self, pos_in_percentage)
        ret = self.poco.rclick(pos_in_percentage)
        if sleep_interval:
            time.sleep(sleep_interval)
        else:
            self.poco.wait_stable()
        self.poco.post_action('rclick', self, pos_in_percentage)
        return ret

    @wait
    def double_click(self, focus=None, sleep_interval=None):
        """
        Perform the double click action on the UI element(s) represented by the UI proxy. If this UI proxy represents a set of
        UI elements, the first one in the set is clicked and the anchor point of the UI element is used as the default
        one. It is also possible to click another point offset by providing ``focus`` argument.

        See ``CoordinateSystem`` for more details.

        Args:
            focus (2-:obj:`tuple`/2-:obj:`list`/:obj:`str`): an offset point (x, y) from the top left corner of the UI
             element(s), values must be in range of 0~1. This argument can be also specified by 'anchor' or 'center'.
             'Center' means to double click the center of bounding box of UI element. 
            sleep_interval: number of seconds to wait after this action. Default is None which is the default sleep
             interval. This value can be configured by Poco initialization. See configuration at poco
             :py:class:`initialization <poco.pocofw.Poco>` for more details.

        Raises:
            PocoNoSuchNodeException: raised when the UI element does not exist
        """

        focus = focus or self._focus or 'anchor'
        pos_in_percentage = self.get_position(focus)
        self.poco.pre_action('double_click', self, pos_in_percentage)
        ret = self.poco.double_click(pos_in_percentage)
        if sleep_interval:
            time.sleep(sleep_interval)
        else:
            self.poco.wait_stable()
        self.poco.post_action('double_click', self, pos_in_percentage)
        return ret

    @wait
    def long_click(self, duration=2.0):
        """
        Perform the long click action on the UI element(s) represented by the UI proxy. If this UI proxy represents a 
        set of UI elements, the first one in the set is clicked and the anchor point of the UI element is used as the 
        default one. Similar to click but press the screen for the given time interval and then release.
    
        Args:
            duration (:py:obj:`float`): whole action duration.
        
        Return:
            the same as :py:meth:`poco.pocofw.Poco.long_click`, depending on poco agent implementation.
        """

        try:
            duration = float(duration)
        except ValueError:
            raise ValueError('Argument `duration` should be <float>. Got {}'.format(repr(duration)))

        pos_in_percentage = self.get_position(self._focus or 'anchor')
        self.poco.pre_action('long_click', self, pos_in_percentage)
        ret = self.poco.long_click(pos_in_percentage, duration)
        self.poco.post_action('long_click', self, pos_in_percentage)
        return ret

    @wait
    def swipe(self, direction, focus=None, duration=0.5):
        """
        Perform a swipe action given by the direction from this UI element. For notices and limitations see
        :py:meth:`.click() <poco.proxy.UIObjectProxy.click>`. 

        Args:
            direction (2-:obj:`tuple`/2-:obj:`list`/:obj:`str`):  coordinates (x, y) in NormalizedCoordinate system, it
             can be also specified as  'up', 'down', 'left', 'right'. Swipe 'up' is equivalent to [0, -0.1], swipe
             'down' is equivalent to [0, 0.1], swipe 'left' is equivalent to [-0.1, 0] and swipe 'right' is equivalent
             to [0.1, 0]
            focus (2-:obj:`tuple`/2-:obj:`list`/:obj:`str`): see :py:meth:`.click() <poco.proxy.UIObjectProxy.click>`
             for more details
            duration (:py:obj:`float`): time interval in which the action is performed

        Raises:
            PocoNoSuchNodeException: raised when the UI element does not exist
        """

        try:
            duration = float(duration)
        except ValueError:
            raise ValueError('Argument `duration` should be <float>. Got {}'.format(repr(duration)))

        focus = focus or self._focus or 'anchor'
        dir_vec = self._direction_vector_of(direction)
        origin = self.get_position(focus)
        self.poco.pre_action('swipe', self, (origin, dir_vec))
        ret = self.poco.swipe(origin, direction=dir_vec, duration=duration)
        self.poco.post_action('swipe', self, (origin, dir_vec))
        return ret

    def drag_to(self, target, duration=2.0):
        """
        Similar to swipe action, but the end point is provide by a UI proxy or by fixed coordinates.

        Args:
            target (:py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`): a UI proxy or 2-list/2-tuple coordinates
             (x, y) in NormalizedCoordinate system
            duration (:py:obj:`float`): time interval in which the action is performed

        Raises:
            PocoNoSuchNodeException: raised when the UI element does not exist
        """

        try:
            duration = float(duration)
        except ValueError:
            raise ValueError('Argument `duration` should be <float>. Got {}'.format(repr(duration)))

        if type(target) in (list, tuple):
            target_pos = target
        else:
            target_pos = target.get_position()
        origin_pos = self.get_position()
        dir_ = [target_pos[0] - origin_pos[0], target_pos[1] - origin_pos[1]]
        return self.swipe(dir_, duration=duration)

    def scroll(self, direction='vertical', percent=0.6, duration=2.0):
        """
        Simply touch down from point A and move to point B then release up finally. This action is performed within
        specific motion range and duration.

        Args:
            direction (:py:obj:`str`): scrolling direction. "vertical" or "horizontal"
            percent (:py:obj:`float`): scrolling distance percentage of selected UI height or width according to
             direction
            duration (:py:obj:`float`): time interval in which the action is performed

        Raises:
            PocoNoSuchNodeException: raised when the UI element does not exist
        """

        if direction not in ('vertical', 'horizontal'):
            raise ValueError('Argument `direction` should be one of "vertical" or "horizontal". Got {}'
                             .format(repr(direction)))

        focus1 = self._focus or [0.5, 0.5]
        focus2 = list(focus1)
        half_distance = percent / 2
        if direction == 'vertical':
            focus1[1] += half_distance
            focus2[1] -= half_distance
        else:
            focus1[0] += half_distance
            focus2[0] -= half_distance

        return self.focus(focus1).drag_to(self.focus(focus2), duration=duration)

    def pinch(self, direction='in', percent=0.6, duration=2.0, dead_zone=0.1):
        """
        Squeezing or expanding 2 fingers on this UI with given motion range and duration.

        Args:
            direction (:py:obj:`str`): pinching direction, only "in" or "out". "in" for squeezing, "out" for expanding
            percent (:py:obj:`float`): squeezing range from or expanding range to of the bounds of the UI
            duration (:py:obj:`float`): time interval in which the action is performed
            dead_zone (:py:obj:`float`): pinching inner circle radius. should not be greater than ``percent``

        Raises:
            PocoNoSuchNodeException: raised when the UI element does not exist
        """

        if direction not in ('in', 'out'):
            raise ValueError('Argument `direction` should be one of "in" or "out". Got {}'.format(repr(direction)))
        if dead_zone >= percent:
            raise ValueError('Argument `dead_zone` should not be greater than `percent`. dead_zoon={}, percent={}'
                             .format(repr(dead_zone), repr(percent)))

        w, h = self.get_size()
        x, y = self.get_position()
        # focus = self._focus or [0.5, 0.5]
        tracks = make_pinching(direction, [x, y], [w, h], percent, dead_zone, duration)
        speed = math.sqrt(w * h) * (percent - dead_zone) / 2 / duration

        # 速度慢的时候，精度适当要提高，这样有助于控制准确
        ret = self.poco.apply_motion_tracks(tracks, accuracy=speed * 0.03)
        return ret

    def pan(self, direction, duration=2.0):
        raise NotImplementedError

    def start_gesture(self):
        """
        Start a gesture action. This method will return a :py:class:`PendingGestureAction
        <poco.gesture.PendingGestureAction>` object which is able to generate decomposed gesture steps. You can invoke
        ``.to`` and ``.hold`` any times in a chain. See the following example.

        Examples:
            ::

                poco = Poco(...)
                ui1 = poco('xxx')
                ui2 = poco('yyy')

                # touch down on ui1 and hold for 1s
                # then drag to ui2 and hold for 1s
                # finally release(touch up)
                ui1.start_gesture().hold(1).to(ui2).hold(1).up()

        .. note:: always starts touching down at the position of current UI object.

        Returns:
            :py:class:`PendingGestureAction <poco.gesture.PendingGestureAction>`: an object for building serialized
            gesture action.
        """

        return PendingGestureAction(self.poco, self)

    def focus(self, f):
        """
        Get a new UI proxy copy with the given focus. Return a new UI proxy object as the UI proxy is immutable.

        Args:
            f (2-:obj:`tuple`/2-:obj:`list`/:obj:`str`): the focus point, it can be specified as 2-list/2-tuple
             coordinates (x, y) in NormalizedCoordinate system or as 'center' or 'anchor'.

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: a new UI proxy object (copy)
        """

        ret = copy.copy(self)
        ret._focus = f
        return ret

    @volatile_attribute
    def get_position(self, focus=None):
        """
        Get the position of the UI elements.

        Args:
            focus: focus point of UI proxy,  see :py:meth:`.focus() <poco.proxy.UIObjectProxy.focus>` for more details

        Returns:
            2-list/2-tuple: coordinates (x, y) in NormalizedCoordinate system

        Raises:
            TypeError: raised when unsupported focus type is specified
        """

        focus = focus or self._focus or 'anchor'
        if focus == 'anchor':
            pos = self.attr('pos')
        elif focus == 'center':
            x, y = self.attr('pos')
            w, h = self.get_size()
            ap_x, ap_y = self.attr("anchorPoint")
            fx, fy = 0.5, 0.5
            pos = [x + w * (fx - ap_x), y + h * (fy - ap_y)]
        elif type(focus) in (list, tuple):
            x, y = self.attr('pos')
            w, h = self.get_size()
            ap_x, ap_y = self.attr("anchorPoint")
            fx, fy = focus
            pos = [x + w * (fx - ap_x), y + h * (fy - ap_y)]
        else:
            raise TypeError('Unsupported focus type {}. '
                            'Only "anchor/center" or 2-list/2-tuple available.'.format(type(focus)))
        return pos

    def _direction_vector_of(self, dir_):
        if dir_ == 'up':
            dir_vec = [0, -0.1]
        elif dir_ == 'down':
            dir_vec = [0, 0.1]
        elif dir_ == 'left':
            dir_vec = [-0.1, 0]
        elif dir_ == 'right':
            dir_vec = [0.1, 0]
        elif type(dir_) in (list, tuple):
            dir_vec = dir_
        else:
            raise TypeError('Unsupported direction type {}. '
                            'Only "up/down/left/right" or 2-list/2-tuple available.'.format(type(dir_)))
        return dir_vec

    def wait(self, timeout=3):
        """
        Block and wait for max given time before the UI element appears.

        Args:
            timeout: maximum waiting time in seconds

        Returns:
            :py:class:`UIObjectProxy <poco.proxy.UIObjectProxy>`: self
        """

        start = time.time()
        while not self.exists():
            self.poco.sleep_for_polling_interval()
            if time.time() - start > timeout:
                break
        return self

    def wait_for_appearance(self, timeout=120):
        """
        Block and wait until the UI element **appears** within the given timeout. When timeout, the
        :py:class:`PocoTargetTimeout <poco.exceptions.PocoTargetTimeout>` is raised.

        Args:
            timeout: maximum waiting time in seconds

        Raises:
            PocoTargetTimeout: when timeout
        """

        start = time.time()
        while not self.exists():
            self.poco.sleep_for_polling_interval()
            if time.time() - start > timeout:
                raise PocoTargetTimeout('appearance', self)

    def wait_for_disappearance(self, timeout=120):
        """
        Block and wait until the UI element **disappears** within the given timeout.

        Args:
            timeout: maximum waiting time in seconds

        Raises:
            PocoTargetTimeout: when timeout
        """

        start = time.time()
        while self.exists():
            self.poco.sleep_for_polling_interval()
            if time.time() - start > timeout:
                raise PocoTargetTimeout('disappearance', self)

    @refresh_when(PocoTargetRemovedException)
    def attr(self, name):
        """
        Retrieve the attribute of UI element by given attribute name. Return None if attribute does not exist.
        If attribute type is :obj:`str`, it is encoded to utf-8 as :obj:`str` in Python2.7.

        Args:
            name: 
                attribute name, it can be one of the following or any other customized type implemented by SDK

                - visible: whether or not it is visible to user
                - text: string value of the UI element
                - type: the type name of UI element from remote runtime
                - pos: the position of the UI element
                - size: the percentage size [width, height] in range of 0~1 according to the screen
                - name: the name of UI element
                - ...: other sdk implemented attributes

        Returns:
            None if no such attribute or its value is None/null/nil/etc. Otherwise the attribute value is returned. The
            returned value type is json serializable. In both py2 and py3, if the attribute value in remote is a
            text-like object, the return value type will be :obj:`str`.

        Raises:
            PocoNoSuchNodeException: when the UI element does not exists

        .. note:: Exception :py:class:`NodeHasBeenRemovedException` is caught automatically.
        
        See Also:
            :py:meth:`UI element attributes in poco sdk definition <poco.sdk.AbstractNode.AbstractNode.getAttr>`.
        """

        # to optimize performance speed, retrieve only the first matched element.
        # 优化速度，只选择第一个匹配到的节点
        nodes = self._do_query(multiple=False)
        val = self.poco.agent.hierarchy.getAttr(nodes, name)
        if six.PY2 and isinstance(val, six.text_type):
            # 文本类型的属性值，只在python2里encode成utf-8的str，python3保持str类型
            # 这是为了在写代码的时候，无论py2/3始终可以像下面这样写
            # node.attr('text') == '节点属性值'
            val = val.encode('utf-8')
        return val

    @refresh_when(PocoTargetRemovedException)
    def setattr(self, name, val):
        """
        Change the attribute value of the UI element. Not all attributes can be casted to text. If changing the
        immutable attributes or attributes which do not exist, the InvalidOperationException exception is raised.

        Args:
            name: attribute name
            val: new attribute value to cast

        Raises:
            InvalidOperationException: when it fails to set the attribute on UI element
        """

        nodes = self._do_query(multiple=False)
        try:
            return self.poco.agent.hierarchy.setAttr(nodes, name, val)
        except UnableToSetAttributeException as e:
            raise InvalidOperationException('"{}" of "{}"'.format(str(e), self))

    @volatile_attribute
    def exists(self):
        """
        Test whether the UI element is in the hierarchy. Similar to :py:meth:`.attr('visible')
        <poco.proxy.UIObjectProxy.attr>`.

        Returns:
            bool: True if exists otherwise False
        """

        try:
            return self.attr('visible')
        except (PocoTargetRemovedException, PocoNoSuchNodeException):
            return False

    def get_text(self):
        """
        Get the text attribute of the UI element. Return None if no such attribute. Similar to :py:meth:`.attr('text')
        <poco.proxy.UIObjectProxy.attr>`.

        Returns:
            :obj:`str`: None if the UI element does not have the text element, otherwise the utf-8 encoded text value.
            In both py2 and py3, the return value type will be :obj:`str`.
        """

        text = self.attr('text')
        return text

    def set_text(self, text):
        """
        Set the text attribute of the UI element. If the UI element does not support mutation, an exception is raised

        Args:
            text: the text value to be set

        Raises:
            InvalidOperationException: when unable to mutate text value of the UI element
        """

        return self.setattr('text', text)

    def get_name(self):
        """
        Get the UI element name attribute

        Returns:
            :obj:`str`: UI element name attribute
        """

        return self.attr('name')

    @volatile_attribute
    def get_size(self):
        """
        Get the UI element size in ``NormalizedCoordinate`` system.

        Returns:
            2-:obj:`list`: size [width, height] in range of 0 ~ 1.
        """

        return self.attr('size')

    @volatile_attribute
    def get_bounds(self):
        """
        Get the parameters of bounding box of the UI element.

        Returns:
            :obj:`list` <:obj:`float`>: 4-list (top, right, bottom, left) coordinates related to the edge of screen in
            NormalizedCoordinate system
        """

        size = self.get_size()
        top_left = self.get_position([0, 0])

        # t, r, b, l
        bounds = [top_left[1], top_left[0] + size[0], top_left[1] + size[1], top_left[0]]
        return bounds

    def __str__(self):
        if six.PY2:
            return 'UIObjectProxy of "{}"'.format(query_expr(self.query)).encode("utf-8")
        else:
            return 'UIObjectProxy of "{}"'.format(query_expr(self.query))

    if six.PY2:
        def __unicode__(self):
            return 'UIObjectProxy of "{}"'.format(query_expr(self.query))

    __repr__ = __str__

    @property
    def nodes(self):
        """
        Readonly property accessing the UI element(s) in the remote runtime.
        """

        return self._do_query()

    def invalidate(self):
        """
        Clear the flag to indicate to re-query or re-select the UI element(s) from hierarchy.
        """

        self._evaluated = False
        self._nodes = None

    def _do_query(self, multiple=True, refresh=False):
        if not self._evaluated or refresh:
            self._nodes = self.poco.agent.hierarchy.select(self.query, multiple)
            if len(self._nodes) == 0:
                raise PocoNoSuchNodeException(self)
            self._evaluated = True
            self._query_multiple = multiple
        return self._nodes



# coding=utf-8
__author__ = 'lxn3032'


class ScreenInterface(object):
    """
    This is one of the main communication interfaces. This interface ensures the ability for accessing the rendering
    the results presented on screen of target device. Note that rendering results are very often not used in automated
    testing directly. Following methods definitions can assist to obtain the information about the app.
    """

    def getScreen(self, width):
        """
        Take the screenshot of the target device screen or target app's window

        Args:
            width (:obj:`int`): expected width of the screenshot in pixels

        Returns:
            2-:obj:`list` (:obj:`str`, :obj:`str`):
                - b64img: base64 encoded screen data
                - format: screen data format (png/jpg/etc.)
        """

        raise NotImplementedError

    def getPortSize(self):
        """
        Get the real resolution of the screen in pixels.

        Returns:
            2-:obj:`list` (:obj:`float`, :obj:`float`): width and height in pixels
        """

        raise NotImplementedError



# coding=utf-8


class HierarchyInterface(object):
    """
    This is one of the main communication interfaces. This interface defines hierarchy-related behaviour assembling
    from :py:class:`IDumper <poco.sdk.AbstractDumper.IDumper>`, :py:class:`Attributor <poco.sdk.Attributor>` and 
    :py:class:`Selector <poco.sdk.Selector>`. The hierarchy mentioned is the real hierarchy on target app runtime,
    e.g. a game UI hierarchy in its runtime.
    """

    def select(self, query, multiple):
        """
        Select UI element(s) matching the given query expression and return the list of selected UI element(s)

        Args:
            query (:obj:`tuple`): query expression,  for the structure specification refer to
             :py:class:`Selector <poco.sdk.Selector>`.
            multiple (:obj:`bool`): whether or not to select multiple elements,  if False, the method terminates \
            immediately once the node is found, otherwise the method travers through all nodes and then terminates

        Returns:
            :obj:`list` : list of UI elements corresponding to the given query expression
        """

        raise NotImplementedError

    def dump(self):
        """
        Get the UI hierarchy with its origin structure and attributes, then store the structure and attributes  into
        a json serializable dictionary.

        Returns:
            :obj:`dict` : dict representing the hierarchy structure. Structure specification refers to \
             :py:class:`IDumper <poco.sdk.AbstractDumper.IDumper>`.
        """

        raise NotImplementedError

    def getAttr(self, nodes, name):
        """
        Get attribute of UI element.

        Args:
            nodes: UI element or list of UI elements, if there is a list of UI elements provided, then only the \
            first UI element will be used
            name (:obj:`str`): attribute name
        """

        raise NotImplementedError

    def setAttr(self, nodes, name, value):
        """
        Set attribute of UI element.

        Args:
            nodes: UI element or list of UI elements, if there is a list of UI elements provided, then only the \
            first UI element will be used
            name (:obj:`str`): attribute name
            value: new value to be set.

        Raises:
            UnableToSetAttributeException: raised when：

                * fails to set attributes on given UI element
                * the  engine does not support mutating attributes
                * developer does not allow to change the attribute value by implementation
        """

        raise NotImplementedError



# coding=utf-8

from poco.sdk.AbstractDumper import AbstractDumper
from poco.sdk.exceptions import InvalidSurfaceException
from poco.drivers.windows.sdk.WindowsUINode import WindowsUINode


class WindowsUIDumper(AbstractDumper):

    def __init__(self, root):
        self.RootControl = root
        self.RootHeight = self.RootControl.BoundingRectangle[3] - self.RootControl.BoundingRectangle[1]
        self.RootWidth = self.RootControl.BoundingRectangle[2] - self.RootControl.BoundingRectangle[0]
        self.RootLeft = self.RootControl.BoundingRectangle[0]
        self.RootTop = self.RootControl.BoundingRectangle[1]
        if self.RootWidth == 0 or self.RootHeight == 0:
                raise InvalidSurfaceException(self, "You may have minimized or closed your window or the window is too small!")

    def getRoot(self):
        return WindowsUINode(self.RootControl, self)



# coding=utf-8

from functools import wraps

from airtest.core.api import device as current_device
from airtest.core.api import touch, swipe
from airtest.core.helper import device_platform, logwrap
from poco.sdk.interfaces.input import InputInterface

__all__ = ['AirtestInput']


def serializable_adapter(func):
    class ISerializable(object):
        def to_json(self):
            raise NotImplementedError

    @wraps(func)
    def wrapped(*args):
        class PocoUIProxySerializable(ISerializable):
            def __init__(self, obj):
                self.obj = obj

            def to_json(self):
                return repr(self.obj)

        new_args = [PocoUIProxySerializable(a) for a in args]
        return func(*new_args)

    return wrapped


@serializable_adapter
@logwrap
def record_ui(driver, action, ui, args):
    return ui


class AirtestInput(InputInterface):
    def __init__(self):
        super(AirtestInput, self).__init__()
        self.default_touch_down_duration = 0.01
        self._driver = None
        self.use_render_resolution = False
        self.render_resolution = None

    def add_preaction_cb(self, driver):
        self._driver = driver
        self._driver.add_pre_action_callback(record_ui)

    def get_target_pos(self, x, y):
        """
        get real time resolution on device of target (x,y)
        """
        offsetx, offsety, pw, ph = self._get_touch_resolution()
        pos = [x * pw + offsetx, y * ph + offsety]
        return pos

    def _get_touch_resolution(self):
        """
        get real time resolution on device if full screen
         or window size if running in window mode
        """
        if device_platform() == 'Android':
            if self.use_render_resolution:
                if self.render_resolution and len(self.render_resolution) == 4:
                    return self.render_resolution
                else:
                    return current_device().get_render_resolution()
        w, h = current_device().get_current_resolution()
        return 0, 0, w, h

    def setTouchDownDuration(self, duration):
        self.default_touch_down_duration = duration

    def getTouchDownDuration(self):
        return self.default_touch_down_duration

    def click(self, x, y):
        pos = self.get_target_pos(x, y)
        touch(pos, duration=self.default_touch_down_duration)

    def swipe(self, x1, y1, x2, y2, duration=2.0):
        if duration <= 0:
            raise ValueError("Operation duration cannot be less equal 0. Please provide a positive number.")
        direction = x2 - x1, y2 - y1
        pos = self.get_target_pos(x1, y1)
        steps = int(duration * 40) + 1
        swipe(pos, vector=direction, duration=duration, steps=steps)

    def longClick(self, x, y, duration=2.0):
        if duration <= 0:
            raise ValueError("Operation duration cannot be less equal 0. Please provide a positive number.")
        pos = self.get_target_pos(x, y)
        touch(pos, duration=duration)

    def applyMotionEvents(self, events):
        if device_platform() != 'Android':
            raise NotImplementedError

        # Android minitouch/maxtouch only, currently
        from airtest.core.android.touch_methods.base_touch import DownEvent, MoveEvent, UpEvent, SleepEvent

        mes = []
        for e in events:
            t = e[0]
            if t == 'd':
                contact = e[2]
                x, y = e[1]
                pos = self.get_target_pos(x, y)
                me = DownEvent(pos, contact)
            elif t == 'm':
                contact = e[2]
                x, y = e[1]
                pos = self.get_target_pos(x, y)
                me = MoveEvent(pos, contact)
            elif t == 'u':
                contact = e[1]
                me = UpEvent(contact)
            elif t == 's':
                how_long = e[1]
                me = SleepEvent(how_long)
            else:
                raise ValueError('Unknown event type {}'.format(repr(t)))
            mes.append(me)

        current_device().touch_proxy.perform(mes, interval=0)



""" Test base JSON-RPC classes."""
import sys
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from ..base import JSONRPCBaseRequest, JSONRPCBaseResponse


class TestJSONRPCBaseRequest(unittest.TestCase):

    """ Test JSONRPCBaseRequest functionality."""

    def test_data(self):
        request = JSONRPCBaseRequest()
        self.assertEqual(request.data, {})

        with self.assertRaises(ValueError):
            request.data = []

        with self.assertRaises(ValueError):
            request.data = None


class TestJSONRPCBaseResponse(unittest.TestCase):

    """ Test JSONRPCBaseResponse functionality."""

    def test_data(self):
        response = JSONRPCBaseResponse(result="")
        self.assertEqual(response.data, {})

        with self.assertRaises(ValueError):
            response.data = []

        with self.assertRaises(ValueError):
            response.data = None



# coding=utf-8

from airtest_hunter import open_platform, AirtestHunter
from hunter_cli.rpc.client import HunterRpcClient

from poco.pocofw import Poco
from poco.agent import PocoAgent
from poco.utils.airtest.input import AirtestInput
from poco.utils.airtest.screen import AirtestScreen
from poco.utils.hrpc.hierarchy import RemotePocoHierarchy
from poco.utils.hunter.command import HunterCommand
from poco.utils import six

__all__ = ['NeteasePoco']
__author__ = 'lxn3032'


class NeteasePocoAgent(PocoAgent):
    def __init__(self, hunter):
        client = HunterRpcClient(hunter)
        client.set_timeout(25)
        remote_poco = client.remote('poco-uiautomation-framework-2')

        # hierarchy
        dumper = remote_poco.dumper
        selector = remote_poco.selector
        attributor = remote_poco.attributor
        hierarchy = RemotePocoHierarchy(dumper, selector, attributor)

        # input
        input = AirtestInput()

        # screen
        screen = AirtestScreen()

        # command
        command = HunterCommand(hunter)

        super(NeteasePocoAgent, self).__init__(hierarchy, input, screen, command)
        self._rpc_client = client


class NeteasePoco(Poco):
    def __init__(self, process, hunter=None, **options):
        apitoken = open_platform.get_api_token(process)
        if hunter:
            self._hunter = hunter
        else:
            self._hunter = AirtestHunter(apitoken, process)
        agent = NeteasePocoAgent(self._hunter)
        super(NeteasePoco, self).__init__(agent, **options)
        self._last_proxy = None
        self.screenshot_each_action = False
        if 'screenshot_each_action' in options:
            self.screenshot_each_action = options['screenshot_each_action']

    def on_pre_action(self, action, ui, args):
        if self.screenshot_each_action:
            # airteset log用
            from airtest.core.api import snapshot
            msg = repr(ui)
            if not isinstance(msg, six.text_type):
                msg = msg.decode('utf-8')
            snapshot(msg=msg)



# coding=utf-8
__author__ = 'lxn3032'

import warnings


class InputInterface(object):
    """
    This is one of the main communication interfaces. This interface ensures the ability for simulated input on target
    device. So far, the interface supports only some basic methods definitions. The motion event will be added in future
    to provide full support for mobile devices.
    
    All coordinates are in NormalizedCoordinate system, see ``NormalizedCoordinate`` for more details.
    """

    def click(self, x, y):
        """
        Perform click action as simulated input on target device. Coordinates arguments are all in range of 0~1.

        Args:
            y (:obj:`float`): y-coordinate
            x (:obj:`float`): x-coordinate
        """

        raise NotImplementedError

    def swipe(self, x1, y1, x2, y2, duration):
        """
        Perform swipe action as simulated input on target device from point A to B within given time interval to
        perform the action. Coordinates  arguments are all in range of 0~1.

        Args:
            x1 (:obj:`float`): x-coordinate of the start point
            y1 (:obj:`float`): y-coordinate of the start point
            x2 (:obj:`float`): x-coordinate of the end point
            y2 (:obj:`float`): y-coordinate of the end point
            duration (:obj:`float`): time interval to perform the swipe action
        """

        raise NotImplementedError

    def longClick(self, x, y, duration):
        """
        Perform press action as simulated input on target device within given seconds. Coordinates arguments are
        all in range of 0~1.

        Args:
            x (:obj:`float`): x-coordinate
            y (:obj:`float`): y-coordinate
            duration (:obj:`float`): time interval to perform the action
        """

        raise NotImplementedError

    def setTouchDownDuration(self, duration):
        """
        Set touch down duration for each click operation.

        Args:
            duration (:obj:`float`): the duration in seconds
        """

        warnings.warn("This implementation of poco does not support changing default touch down duration. "
                      "Setting touch down duration to {}s takes no effects.".format(duration))

    def getTouchDownDuration(self):
        """
        Get touch down duration for each click operation. Each implementation should provide a default value.

        Return:
            :obj:`float`: the touch down duration in seconds
        """

        raise NotImplementedError

    def keyevent(self, keycode):
        """
        Send a key event to target device.

        Args:
            keycode (:obj:`int` or :obj:`char`): Ascii key code
        """

        raise NotImplementedError

    def applyMotionEvents(self, events):
        """
        Apply a motion event on the device touch panel

        Args:
            events (:py:obj:`list`): list of events. each event is MotionEvent data
             (``['u/d/m/s', (x, y), contact_id]``)
        """

        raise NotImplementedError



# coding=utf-8

import errno
import socket
import select
import uuid

from poco.sdk.std.transport import Transport
from poco.sdk.std.protocol import SimpleProtocolFilter
from poco.utils import six

if six.PY3:
    from queue import Queue, Empty
else:
    from Queue import Queue, Empty


class ConnectionReset(Exception):
    pass


class Connection(object):
    def __init__(self, cid, sock, endpoint, RX_SIZE=65536):
        super(Connection, self).__init__()
        self.cid = cid
        self.sock = sock
        self.endpoint = endpoint
        self.p = SimpleProtocolFilter()
        self.RX_SIZE = RX_SIZE

    def send(self, packet):
        data = self.p.pack(packet)
        self.sock.sendall(data)

    def recv(self):
        rxdata = ''
        try:
            rxdata = self.sock.recv(self.RX_SIZE)
        except socket.error as e:
            if e.errno in (errno.ECONNRESET, ):
                raise ConnectionReset

        if not rxdata:
            self.close()
            raise ConnectionReset
        else:
            for packet in self.p.input(rxdata):
                yield packet

    def close(self):
        try:
            self.sock.close()
        except socket.error:
            pass

    def get_socket_object(self):
        return self.sock


class TcpSocket(Transport):
    def __init__(self, RX_SIZE=65536):
        super(TcpSocket, self).__init__()
        # active socket object
        self.s = None
        self.connections = {}  # sock -> Connection
        self.connections_endpoints = {}  # endpoint -> Connection

        self.rq = Queue()
        self.RX_SIZE = RX_SIZE

    def connect(self, endpoint):
        if endpoint in self.connections_endpoints:
            raise RuntimeError("Already connected to {}".format(endpoint))

        c = socket.socket()
        c.connect(endpoint)
        cid = six.text_type(uuid.uuid4())
        conn = Connection(cid, c, endpoint, self.RX_SIZE)
        self.connections[c] = conn
        self.connections_endpoints[endpoint] = conn

    def disconnect(self, endpoint=None):
        if endpoint is not None:
            conn = self.connections_endpoints.pop(endpoint, None)
            if conn:
                self.connections.pop(conn.get_socket_object(), None)
                conn.close()
        else:
            for _, conn in self.connections_endpoints.items():
                conn.close()
            self.connections_endpoints = {}
            self.connections = {}

    def bind(self, endpoint):
        if self.s is not None:
            raise RuntimeError("Already bound at {}".format(self.s.getsockname()))

        ip, port = endpoint
        if ip in ('', '*', '0'):
            ip = '0.0.0.0'
        self.s = socket.socket()
        self.s.bind((ip, port))
        self.s.listen(10)
        print('server listens on ("{}", {}) transport socket'.format(ip, port))

    def update(self, timeout=0.002):
        rlist = []
        if self.s is not None:
            rlist.append(self.s)
        rlist.extend(self.connections.keys())
        r, _, _ = select.select(rlist, [], [], timeout)
        for c in r:
            if c is self.s:
                client_sock, endpoint = self.s.accept()
                print('accept from: {}'.format(endpoint))
                cid = six.text_type(uuid.uuid4())
                conn = Connection(cid, client_sock, endpoint, self.RX_SIZE)
                self.connections[client_sock] = conn
                self.connections_endpoints[endpoint] = conn
            else:
                conn = self.connections.get(c)
                if not conn:
                    continue

                try:
                    for packet in conn.recv():
                        self.rq.put((conn.cid, packet))
                except ConnectionReset:
                    self.connections.pop(c)

        return self.recv()

    def recv(self):
        try:
            return self.rq.get(False)
        except Empty:
            return None, None

    def send(self, cid, packet):
        if cid is None:
            # broadcast
            for _, conn in self.connections.items():
                conn.send(packet)
        else:
            conn = self.get_connection(cid)
            if conn:
                conn.send(packet)

    def get_connection(self, cid):
        for sock, conn in self.connections.items():
            if conn.cid == cid:
                return conn
        return None

    def __str__(self):
        return 'Tcp connection(s) at {}'.format(self.connections_endpoints.keys())

    __repr__ = __str__



# coding=utf-8
from __future__ import absolute_import

from airtest.core.device import Device


class VirtualDevice(Device):
    def __init__(self, ip):
        super(VirtualDevice, self).__init__()
        self.ip = ip

    @property
    def uuid(self):
        return 'virtual-device'

    def get_current_resolution(self):
        return [1920, 1080]

    def get_ip_address(self):
        return self.ip



# coding=utf-8
from setuptools import setup, find_packages


def parse_requirements(filename='requirements.txt'):
    """ load requirements from a pip requirements file. (replacing from pip.req import parse_requirements)"""
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


setup(
    name='pocoui',
    version='1.0.82',
    keywords="poco, automation test, ui automation",
    description='Poco cross-engine UI automated test framework.',
    long_description='Poco cross-engine UI automated test framework. 2018 present by NetEase Games',
    packages=find_packages(),
    include_package_data=True,
    install_requires=parse_requirements(),
    license='Apache License 2.0',

    author='Netease Games',
    author_email='lxn3032@corp.netease.com, gzliuxin@corp.netease.com',
    url='https://github.com/AirtestProject/Poco',
)



# coding=utf-8

import base64
import zlib

from poco.sdk.interfaces.screen import ScreenInterface
from poco.utils.simplerpc.utils import sync_wrapper


class StdScreen(ScreenInterface):
    def __init__(self, client):
        super(StdScreen, self).__init__()
        self.client = client

    @sync_wrapper
    def _getScreen(self, width):
        return self.client.call("Screenshot", width)

    def getScreen(self, width):
        b64, fmt = self._getScreen(width)
        if fmt.endswith('.deflate'):
            fmt = fmt[:-len('.deflate')]
            imgdata = base64.b64decode(b64)
            imgdata = zlib.decompress(imgdata)
            b64 = base64.b64encode(imgdata)
        return b64, fmt

    @sync_wrapper
    def getPortSize(self):
        return self.client.call("GetScreenSize")


# coding=utf-8

import json
import time

from poco.utils import six


class RpcRemoteException(Exception):
    pass


class StdRpcEndpointController(object):
    def __init__(self, transport, reactor):
        super(StdRpcEndpointController, self).__init__()
        self.transport = transport
        self.reactor = reactor

    def deserialize(self, data):
        if six.PY3 and not isinstance(data, six.text_type):
            data = data.decode('utf-8')
        return json.loads(data)

    def serialize(self, packet):
        return json.dumps(packet)

    def serve_forever(self):
        while True:
            cid, data = self.transport.update()
            if data:
                packet = self.deserialize(data)
                if 'method' in packet:
                    result = self.reactor.handle_request(packet)
                    sres = self.serialize(result)
                    self.transport.send(cid, sres)
                else:
                    self.reactor.handle_response(packet)

    def call(self, method, *args, **kwargs):
        req = self.reactor.build_request(method, *args, **kwargs)
        rid = req['id']
        sreq = self.serialize(req)
        self.transport.send(None, sreq)
        while True:
            time.sleep(0.004)
            res = self.reactor.get_result(rid)
            if res is not None:
                if 'result' in res:
                    return res['result']

                if 'error' in res:
                    raise RpcRemoteException(res['error']['message'])

                raise RuntimeError('Invalid response from {}. Got {}'.format(self.transport, res))


# coding=utf-8

import time
from poco.exceptions import PocoNoSuchNodeException
from poco.drivers.unity3d.test.tutorial.case import TutorialCase


class InvalidOperationExceptionTutorial(TutorialCase):
    def runTest(self):
        node = self.poco('not existed node')  # select will never raise any exceptions
        try:
            node.click()
        except PocoNoSuchNodeException:
            print('oops!')
            time.sleep(1)

        try:
            node.attr('text')
        except PocoNoSuchNodeException:
            print('oops!')
            time.sleep(1)

        print(node.exists())  # => False. this method will not raise
        time.sleep(0.2)


if __name__ == '__main__':
    import pocounit
    pocounit.main()
